@@ -16,9 +16,11 @@
 """Image V2 Action Implementations"""
 
 import argparse
+from base64 import b64encode
 import logging
 
 from glanceclient.common import utils as gc_utils
+from openstack.image import image_signer
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
@@ -183,6 +185,22 @@ class CreateImage(command.ShowOne):
             help=_("Force image creation if volume is in use "
                    "(only meaningful with --volume)"),
         )
+        parser.add_argument(
+            '--sign-key-path',
+            metavar="<sign-key-path>",
+            default=[],
+            help=_("Sign the image using the specified private key. "
+                   "Only use in combination with --sign-cert-id")
+        )
+        parser.add_argument(
+            '--sign-cert-id',
+            metavar="<sign-cert-id>",
+            default=[],
+            help=_("The specified certificate UUID is a reference to "
+                   "the certificate in the key manager that corresponds "
+                   "to the public key and is used for signature validation. "
+                   "Only use in combination with --sign-key-path")
+        )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
             "--protected",
@@ -335,6 +353,46 @@ class CreateImage(command.ShowOne):
                 parsed_args.project_domain,
             ).id
 
+        # sign an image using a given local private key file
+        if parsed_args.sign_key_path or parsed_args.sign_cert_id:
+            if not parsed_args.file:
+                msg = (_("signing an image requires the --file option, "
+                         "passing files via stdin when signing is not "
+                         "supported."))
+                raise exceptions.CommandError(msg)
+            if (len(parsed_args.sign_key_path) < 1 or
+                    len(parsed_args.sign_cert_id) < 1):
+                msg = (_("'sign-key-path' and 'sign-cert-id' must both be "
+                         "specified when attempting to sign an image."))
+                raise exceptions.CommandError(msg)
+            else:
+                sign_key_path = parsed_args.sign_key_path
+                sign_cert_id = parsed_args.sign_cert_id
+                signer = image_signer.ImageSigner()
+                try:
+                    pw = utils.get_password(
+                        self.app.stdin,
+                        prompt=("Please enter private key password, leave "
+                                "empty if none: "),
+                        confirm=False)
+                    if not pw or len(pw) < 1:
+                        pw = None
+                    signer.load_private_key(
+                        sign_key_path,
+                        password=pw)
+                except Exception:
+                    msg = (_("Error during sign operation: private key could "
+                             "not be loaded."))
+                    raise exceptions.CommandError(msg)
+
+                signature = signer.generate_signature(fp)
+                signature_b64 = b64encode(signature)
+                kwargs['img_signature'] = signature_b64
+                kwargs['img_signature_certificate_uuid'] = sign_cert_id
+                kwargs['img_signature_hash_method'] = signer.hash_method
+                if signer.padding_method:
+                    kwargs['img_signature_key_type'] = signer.padding_method
+
         # If a volume is specified.
         if parsed_args.volume:
             volume_client = self.app.client_manager.volume
@@ -439,6 +497,13 @@ class ListImage(command.Lister):
             default=False,
             help=_("List only private images"),
         )
+        public_group.add_argument(
+            "--community",
+            dest="community",
+            action="store_true",
+            default=False,
+            help=_("List only community images"),
+        )
         public_group.add_argument(
             "--shared",
             dest="shared",
@@ -464,6 +529,12 @@ class ListImage(command.Lister):
             default=None,
             help=_("Filter images based on status.")
         )
+        parser.add_argument(
+            '--tag',
+            metavar='<tag>',
+            default=None,
+            help=_('Filter images based on tag.'),
+        )
         parser.add_argument(
             '--long',
             action='store_true',
@@ -510,6 +581,8 @@ class ListImage(command.Lister):
             kwargs['public'] = True
         if parsed_args.private:
             kwargs['private'] = True
+        if parsed_args.community:
+            kwargs['community'] = True
         if parsed_args.shared:
             kwargs['shared'] = True
         if parsed_args.limit:
@@ -521,6 +594,8 @@ class ListImage(command.Lister):
             kwargs['name'] = parsed_args.name
         if parsed_args.status:
             kwargs['status'] = parsed_args.status
+        if parsed_args.tag:
+            kwargs['tag'] = parsed_args.tag
         if parsed_args.long:
             columns = (
                 'ID',
@@ -577,7 +652,7 @@ class ListImage(command.Lister):
                 property_field='properties',
             )
 
-        data = utils.sort_items(data, parsed_args.sort)
+        data = utils.sort_items(data, parsed_args.sort, str)
 
         return (
             column_headers,
@@ -591,6 +666,39 @@ class ListImage(command.Lister):
         )
 
 
+class ListImageProjects(command.Lister):
+    _description = _("List projects associated with image")
+
+    def get_parser(self, prog_name):
+        parser = super(ListImageProjects, self).get_parser(prog_name)
+        parser.add_argument(
+            "image",
+            metavar="<image>",
+            help=_("Image (name or ID)"),
+        )
+        common.add_project_domain_option_to_parser(parser)
+        return parser
+
+    def take_action(self, parsed_args):
+        image_client = self.app.client_manager.image
+        columns = (
+            "Image ID",
+            "Member ID",
+            "Status"
+        )
+
+        image_id = utils.find_resource(
+            image_client.images,
+            parsed_args.image).id
+
+        data = image_client.image_members.list(image_id)
+
+        return (columns,
+                (utils.get_item_properties(
+                    s, columns,
+                ) for s in data))
+
+
 class RemoveProjectImage(command.Command):
     _description = _("Disassociate project with image")
 
@@ -649,6 +757,12 @@ class SaveImage(command.Command):
         )
         data = image_client.images.data(image.id)
 
+        if data.wrapped is None:
+            msg = _('Image %s has no data.') % image.id
+            LOG.error(msg)
+            self.app.stdout.write(msg + '\n')
+            raise SystemExit
+
         gc_utils.save_image(data, parsed_args.file)
 
 
@@ -1024,9 +1138,7 @@ class UnsetImage(command.Command):
 
         if parsed_args.properties:
             for k in parsed_args.properties:
-                try:
-                    assert(k in image.keys())
-                except AssertionError:
+                if k not in image:
                     LOG.error(_("property unset failed, '%s' is a "
                                 "nonexistent property "), k)
                     propret += 1
