@@ -16,6 +16,7 @@ from mock import call
 
 from keystoneauth1 import exceptions as ks_exc
 from osc_lib import exceptions
+from osc_lib import utils
 
 from openstackclient.identity.v3 import group
 from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
@@ -41,34 +42,79 @@ class TestGroup(identity_fakes.TestIdentityv3):
 
 class TestGroupAddUser(TestGroup):
 
-    group = identity_fakes.FakeGroup.create_one_group()
-    user = identity_fakes.FakeUser.create_one_user()
+    _group = identity_fakes.FakeGroup.create_one_group()
+    users = identity_fakes.FakeUser.create_users(count=2)
 
     def setUp(self):
         super(TestGroupAddUser, self).setUp()
 
-        self.groups_mock.get.return_value = self.group
-        self.users_mock.get.return_value = self.user
+        self.groups_mock.get.return_value = self._group
+        self.users_mock.get = (
+            identity_fakes.FakeUser.get_users(self.users))
         self.users_mock.add_to_group.return_value = None
 
         self.cmd = group.AddUserToGroup(self.app, None)
 
     def test_group_add_user(self):
         arglist = [
-            self.group.name,
-            self.user.name,
+            self._group.name,
+            self.users[0].name,
         ]
         verifylist = [
-            ('group', self.group.name),
-            ('user', self.user.name),
+            ('group', self._group.name),
+            ('user', [self.users[0].name]),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
         self.users_mock.add_to_group.assert_called_once_with(
-            self.user.id, self.group.id)
+            self.users[0].id, self._group.id)
         self.assertIsNone(result)
 
+    def test_group_add_multi_users(self):
+        arglist = [
+            self._group.name,
+            self.users[0].name,
+            self.users[1].name,
+        ]
+        verifylist = [
+            ('group', self._group.name),
+            ('user', [self.users[0].name, self.users[1].name]),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        calls = [call(self.users[0].id, self._group.id),
+                 call(self.users[1].id, self._group.id)]
+        self.users_mock.add_to_group.assert_has_calls(calls)
+        self.assertIsNone(result)
+
+    @mock.patch.object(group.LOG, 'error')
+    def test_group_add_user_with_error(self, mock_error):
+        self.users_mock.add_to_group.side_effect = [
+            exceptions.CommandError(), None]
+        arglist = [
+            self._group.name,
+            self.users[0].name,
+            self.users[1].name,
+        ]
+        verifylist = [
+            ('group', self._group.name),
+            ('user', [self.users[0].name, self.users[1].name]),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        try:
+            self.cmd.take_action(parsed_args)
+            self.fail('CommandError should be raised.')
+        except exceptions.CommandError as e:
+            msg = "1 of 2 users not added to group %s." % self._group.name
+            self.assertEqual(msg, str(e))
+        msg = ("%(user)s not added to group %(group)s: ") % {
+            'user': self.users[0].name,
+            'group': self._group.name,
+        }
+        mock_error.assert_called_once_with(msg)
+
 
 class TestGroupCheckUser(TestGroup):
 
@@ -100,6 +146,23 @@ class TestGroupCheckUser(TestGroup):
             self.user.id, self.group.id)
         self.assertIsNone(result)
 
+    def test_group_check_user_server_error(self):
+        def server_error(*args):
+            raise ks_exc.http.InternalServerError
+        self.users_mock.check_in_group.side_effect = server_error
+        arglist = [
+            self.group.name,
+            self.user.name,
+        ]
+        verifylist = [
+            ('group', self.group.name),
+            ('user', self.user.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(ks_exc.http.InternalServerError,
+                          self.cmd.take_action, parsed_args)
+
 
 class TestGroupCreate(TestGroup):
 
@@ -257,6 +320,32 @@ class TestGroupDelete(TestGroup):
         self.groups_mock.delete.assert_called_once_with(self.groups[0].id)
         self.assertIsNone(result)
 
+    @mock.patch.object(utils, 'find_resource')
+    def test_delete_multi_groups_with_exception(self, find_mock):
+        find_mock.side_effect = [self.groups[0],
+                                 exceptions.CommandError]
+        arglist = [
+            self.groups[0].id,
+            'unexist_group',
+        ]
+        verifylist = [
+            ('groups', arglist),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        try:
+            self.cmd.take_action(parsed_args)
+            self.fail('CommandError should be raised.')
+        except exceptions.CommandError as e:
+            self.assertEqual('1 of 2 groups failed to delete.',
+                             str(e))
+
+        find_mock.assert_any_call(self.groups_mock, self.groups[0].id)
+        find_mock.assert_any_call(self.groups_mock, 'unexist_group')
+
+        self.assertEqual(2, find_mock.call_count)
+        self.groups_mock.delete.assert_called_once_with(self.groups[0].id)
+
 
 class TestGroupList(TestGroup):
 
@@ -405,34 +494,79 @@ class TestGroupList(TestGroup):
 
 class TestGroupRemoveUser(TestGroup):
 
-    group = identity_fakes.FakeGroup.create_one_group()
-    user = identity_fakes.FakeUser.create_one_user()
+    _group = identity_fakes.FakeGroup.create_one_group()
+    users = identity_fakes.FakeUser.create_users(count=2)
 
     def setUp(self):
         super(TestGroupRemoveUser, self).setUp()
 
-        self.groups_mock.get.return_value = self.group
-        self.users_mock.get.return_value = self.user
+        self.groups_mock.get.return_value = self._group
+        self.users_mock.get = (
+            identity_fakes.FakeUser.get_users(self.users))
         self.users_mock.remove_from_group.return_value = None
 
         self.cmd = group.RemoveUserFromGroup(self.app, None)
 
     def test_group_remove_user(self):
         arglist = [
-            self.group.id,
-            self.user.id,
+            self._group.id,
+            self.users[0].id,
         ]
         verifylist = [
-            ('group', self.group.id),
-            ('user', self.user.id),
+            ('group', self._group.id),
+            ('user', [self.users[0].id]),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
         self.users_mock.remove_from_group.assert_called_once_with(
-            self.user.id, self.group.id)
+            self.users[0].id, self._group.id)
         self.assertIsNone(result)
 
+    def test_group_remove_multi_users(self):
+        arglist = [
+            self._group.name,
+            self.users[0].name,
+            self.users[1].name,
+        ]
+        verifylist = [
+            ('group', self._group.name),
+            ('user', [self.users[0].name, self.users[1].name]),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        calls = [call(self.users[0].id, self._group.id),
+                 call(self.users[1].id, self._group.id)]
+        self.users_mock.remove_from_group.assert_has_calls(calls)
+        self.assertIsNone(result)
+
+    @mock.patch.object(group.LOG, 'error')
+    def test_group_remove_user_with_error(self, mock_error):
+        self.users_mock.remove_from_group.side_effect = [
+            exceptions.CommandError(), None]
+        arglist = [
+            self._group.id,
+            self.users[0].id,
+            self.users[1].id,
+        ]
+        verifylist = [
+            ('group', self._group.id),
+            ('user', [self.users[0].id, self.users[1].id]),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        try:
+            self.cmd.take_action(parsed_args)
+            self.fail('CommandError should be raised.')
+        except exceptions.CommandError as e:
+            msg = "1 of 2 users not removed from group %s." % self._group.id
+            self.assertEqual(msg, str(e))
+        msg = ("%(user)s not removed from group %(group)s: ") % {
+            'user': self.users[0].id,
+            'group': self._group.id,
+        }
+        mock_error.assert_called_once_with(msg)
+
 
 class TestGroupSet(TestGroup):
 
