@@ -193,6 +193,100 @@ class FakeService(object):
         return services
 
 
+class FakeCapability(object):
+    """Fake capability."""
+
+    @staticmethod
+    def create_one_capability(attrs=None):
+        """Create a fake volume backend capability.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes of the Capabilities.
+        :return:
+            A FakeResource object with capability name and attrs.
+        """
+        # Set default attribute
+        capability_info = {
+            "namespace": "OS::Storage::Capabilities::fake",
+            "vendor_name": "OpenStack",
+            "volume_backend_name": "lvmdriver-1",
+            "pool_name": "pool",
+            "driver_version": "2.0.0",
+            "storage_protocol": "iSCSI",
+            "display_name": "Capabilities of Cinder LVM driver",
+            "description": "Blah, blah.",
+            "visibility": "public",
+            "replication_targets": [],
+            "properties": {
+                "compression": {
+                    "title": "Compression",
+                    "description": "Enables compression.",
+                    "type": "boolean"
+                },
+                "qos": {
+                    "title": "QoS",
+                    "description": "Enables QoS.",
+                    "type": "boolean"
+                },
+                "replication": {
+                    "title": "Replication",
+                    "description": "Enables replication.",
+                    "type": "boolean"
+                },
+                "thin_provisioning": {
+                    "title": "Thin Provisioning",
+                    "description": "Sets thin provisioning.",
+                    "type": "boolean"
+                }
+            }
+        }
+
+        # Overwrite default attributes if there are some attributes set
+        capability_info.update(attrs or {})
+
+        capability = fakes.FakeResource(
+            None,
+            capability_info,
+            loaded=True)
+
+        return capability
+
+
+class FakePool(object):
+    """Fake Pools."""
+
+    @staticmethod
+    def create_one_pool(attrs=None):
+        """Create a fake pool.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes of the pool
+        :return:
+            A FakeResource object with pool name and attrs.
+        """
+        # Set default attribute
+        pool_info = {
+            'name': 'host@lvmdriver-1#lvmdriver-1',
+            'storage_protocol': 'iSCSI',
+            'thick_provisioning_support': False,
+            'thin_provisioning_support': True,
+            'total_volumes': 99,
+            'total_capacity_gb': 1000.00,
+            'allocated_capacity_gb': 100,
+            'max_over_subscription_ratio': 200.0,
+        }
+
+        # Overwrite default attributes if there are some attributes set
+        pool_info.update(attrs or {})
+
+        pool = fakes.FakeResource(
+            None,
+            pool_info,
+            loaded=True)
+
+        return pool
+
+
 class FakeVolumeClient(object):
 
     def __init__(self, **kwargs):
@@ -200,6 +294,8 @@ class FakeVolumeClient(object):
         self.volumes.resource_class = fakes.FakeResource(None, {})
         self.extensions = mock.Mock()
         self.extensions.resource_class = fakes.FakeResource(None, {})
+        self.limits = mock.Mock()
+        self.limits.resource_class = fakes.FakeResource(None, {})
         self.volume_snapshots = mock.Mock()
         self.volume_snapshots.resource_class = fakes.FakeResource(None, {})
         self.backups = mock.Mock()
@@ -231,6 +327,10 @@ class FakeVolumeClient(object):
         self.cgsnapshots.resource_class = fakes.FakeResource(None, {})
         self.auth_token = kwargs['token']
         self.management_url = kwargs['endpoint']
+        self.capabilities = mock.Mock()
+        self.capabilities.resource_class = fakes.FakeResource(None, {})
+        self.pools = mock.Mock()
+        self.pools.resource_class = fakes.FakeResource(None, {})
 
 
 class TestVolume(utils.TestCommand):
@@ -1004,3 +1104,101 @@ class FakeQuota(object):
         quota.project_id = quota_attrs['id']
 
         return quota
+
+
+class FakeLimits(object):
+    """Fake limits"""
+
+    def __init__(self, absolute_attrs=None):
+        self.absolute_limits_attrs = {
+            'totalSnapshotsUsed': 1,
+            'maxTotalBackups': 10,
+            'maxTotalVolumeGigabytes': 1000,
+            'maxTotalSnapshots': 10,
+            'maxTotalBackupGigabytes': 1000,
+            'totalBackupGigabytesUsed': 0,
+            'maxTotalVolumes': 10,
+            'totalVolumesUsed': 4,
+            'totalBackupsUsed': 0,
+            'totalGigabytesUsed': 35
+        }
+        absolute_attrs = absolute_attrs or {}
+        self.absolute_limits_attrs.update(absolute_attrs)
+
+        self.rate_limits_attrs = [{
+            "uri": "*",
+            "limit": [
+                {
+                    "value": 10,
+                    "verb": "POST",
+                    "remaining": 2,
+                    "unit": "MINUTE",
+                    "next-available": "2011-12-15T22:42:45Z"
+                },
+                {
+                    "value": 10,
+                    "verb": "PUT",
+                    "remaining": 2,
+                    "unit": "MINUTE",
+                    "next-available": "2011-12-15T22:42:45Z"
+                },
+                {
+                    "value": 100,
+                    "verb": "DELETE",
+                    "remaining": 100,
+                    "unit": "MINUTE",
+                    "next-available": "2011-12-15T22:42:45Z"
+                }
+            ]
+        }]
+
+    @property
+    def absolute(self):
+        for (name, value) in self.absolute_limits_attrs.items():
+            yield FakeAbsoluteLimit(name, value)
+
+    def absolute_limits(self):
+        reference_data = []
+        for (name, value) in self.absolute_limits_attrs.items():
+            reference_data.append((name, value))
+        return reference_data
+
+    @property
+    def rate(self):
+        for group in self.rate_limits_attrs:
+            uri = group['uri']
+            for rate in group['limit']:
+                yield FakeRateLimit(rate['verb'], uri, rate['value'],
+                                    rate['remaining'], rate['unit'],
+                                    rate['next-available'])
+
+    def rate_limits(self):
+        reference_data = []
+        for group in self.rate_limits_attrs:
+            uri = group['uri']
+            for rate in group['limit']:
+                reference_data.append((rate['verb'], uri, rate['value'],
+                                       rate['remaining'], rate['unit'],
+                                       rate['next-available']))
+        return reference_data
+
+
+class FakeAbsoluteLimit(object):
+    """Data model that represents an absolute limit."""
+
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+
+
+class FakeRateLimit(object):
+    """Data model that represents a flattened view of a single rate limit."""
+
+    def __init__(self, verb, uri, value, remain,
+                 unit, next_available):
+        self.verb = verb
+        self.uri = uri
+        self.value = value
+        self.remain = remain
+        self.unit = unit
+        self.next_available = next_available
