@@ -13,9 +13,9 @@
 #
 
 import argparse
+
 import mock
 from mock import call
-
 from osc_lib import exceptions
 from osc_lib import utils
 
@@ -126,15 +126,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=None,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -152,7 +148,6 @@ class TestVolumeCreate(TestVolume):
             '--availability-zone', self.new_volume.availability_zone,
             '--consistency-group', consistency_group.id,
             '--hint', 'k=v',
-            '--multi-attach',
             self.new_volume.name,
         ]
         verifylist = [
@@ -162,7 +157,6 @@ class TestVolumeCreate(TestVolume):
             ('availability_zone', self.new_volume.availability_zone),
             ('consistency_group', consistency_group.id),
             ('hint', {'k': 'v'}),
-            ('multi_attach', True),
             ('name', self.new_volume.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -178,112 +172,50 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=self.new_volume.description,
             volume_type=self.new_volume.volume_type,
-            user_id=None,
-            project_id=None,
             availability_zone=self.new_volume.availability_zone,
             metadata=None,
             imageRef=None,
             source_volid=None,
             consistencygroup_id=consistency_group.id,
-            source_replica=None,
-            multiattach=True,
             scheduler_hints={'k': 'v'},
         )
 
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, data)
 
-    def test_volume_create_user_project_id(self):
-        # Return a project
-        self.projects_mock.get.return_value = self.project
-        # Return a user
-        self.users_mock.get.return_value = self.user
-
+    def test_volume_create_user(self):
         arglist = [
             '--size', str(self.new_volume.size),
-            '--project', self.project.id,
             '--user', self.user.id,
             self.new_volume.name,
         ]
         verifylist = [
             ('size', self.new_volume.size),
-            ('project', self.project.id),
             ('user', self.user.id),
             ('name', self.new_volume.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # In base command class ShowOne in cliff, abstract method take_action()
-        # returns a two-part tuple with a tuple of column names and a tuple of
-        # data to be shown.
-        columns, data = self.cmd.take_action(parsed_args)
-
-        self.volumes_mock.create.assert_called_with(
-            size=self.new_volume.size,
-            snapshot_id=None,
-            name=self.new_volume.name,
-            description=None,
-            volume_type=None,
-            user_id=self.user.id,
-            project_id=self.project.id,
-            availability_zone=None,
-            metadata=None,
-            imageRef=None,
-            source_volid=None,
-            consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
-            scheduler_hints=None,
-        )
-
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, data)
-
-    def test_volume_create_user_project_name(self):
-        # Return a project
-        self.projects_mock.get.return_value = self.project
-        # Return a user
-        self.users_mock.get.return_value = self.user
+        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
+                          parsed_args)
+        self.volumes_mock.create.assert_not_called()
 
+    def test_volume_create_project(self):
         arglist = [
             '--size', str(self.new_volume.size),
-            '--project', self.project.name,
-            '--user', self.user.name,
+            '--project', self.project.id,
             self.new_volume.name,
         ]
         verifylist = [
             ('size', self.new_volume.size),
-            ('project', self.project.name),
-            ('user', self.user.name),
+            ('project', self.project.id),
             ('name', self.new_volume.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        # In base command class ShowOne in cliff, abstract method take_action()
-        # returns a two-part tuple with a tuple of column names and a tuple of
-        # data to be shown.
-        columns, data = self.cmd.take_action(parsed_args)
-
-        self.volumes_mock.create.assert_called_with(
-            size=self.new_volume.size,
-            snapshot_id=None,
-            name=self.new_volume.name,
-            description=None,
-            volume_type=None,
-            user_id=self.user.id,
-            project_id=self.project.id,
-            availability_zone=None,
-            metadata=None,
-            imageRef=None,
-            source_volid=None,
-            consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
-            scheduler_hints=None,
-        )
-
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, data)
+        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
+                          parsed_args)
+        self.volumes_mock.create.assert_not_called()
 
     def test_volume_create_properties(self):
         arglist = [
@@ -310,15 +242,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata={'Alpha': 'a', 'Beta': 'b'},
             imageRef=None,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -352,15 +280,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=image.id,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -394,15 +318,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=image.id,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -430,20 +350,16 @@ class TestVolumeCreate(TestVolume):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.volumes_mock.create.assert_called_once_with(
-            size=None,
+            size=snapshot.size,
             snapshot_id=snapshot.id,
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=None,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -477,15 +393,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=None,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -523,15 +435,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=None,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -578,15 +486,11 @@ class TestVolumeCreate(TestVolume):
             name=self.new_volume.name,
             description=None,
             volume_type=None,
-            user_id=None,
-            project_id=None,
             availability_zone=None,
             metadata=None,
             imageRef=None,
             source_volid=None,
             consistencygroup_id=None,
-            source_replica=None,
-            multiattach=False,
             scheduler_hints=None,
         )
 
@@ -598,40 +502,6 @@ class TestVolumeCreate(TestVolume):
         self.volumes_mock.update_readonly_flag.assert_called_with(
             self.new_volume.id, True)
 
-    def test_volume_create_with_source_replicated(self):
-        self.volumes_mock.get.return_value = self.new_volume
-        arglist = [
-            '--source-replicated', self.new_volume.id,
-            self.new_volume.name,
-        ]
-        verifylist = [
-            ('source_replicated', self.new_volume.id),
-            ('name', self.new_volume.name),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        columns, data = self.cmd.take_action(parsed_args)
-        self.volumes_mock.create.assert_called_once_with(
-            size=None,
-            snapshot_id=None,
-            name=self.new_volume.name,
-            description=None,
-            volume_type=None,
-            user_id=None,
-            project_id=None,
-            availability_zone=None,
-            metadata=None,
-            imageRef=None,
-            source_volid=None,
-            consistencygroup_id=None,
-            source_replica=self.new_volume.id,
-            multiattach=False,
-            scheduler_hints=None,
-        )
-
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, data)
-
     def test_volume_create_without_size(self):
         arglist = [
             self.new_volume.name,
@@ -649,7 +519,6 @@ class TestVolumeCreate(TestVolume):
             '--image', 'source_image',
             '--source', 'source_volume',
             '--snapshot', 'source_snapshot',
-            '--source-replicated', 'source_replicated_volume',
             '--size', str(self.new_volume.size),
             self.new_volume.name,
         ]
@@ -657,7 +526,6 @@ class TestVolumeCreate(TestVolume):
             ('image', 'source_image'),
             ('source', 'source_volume'),
             ('snapshot', 'source_snapshot'),
-            ('source-replicated', 'source_replicated_volume'),
             ('size', self.new_volume.size),
             ('name', self.new_volume.name),
         ]
@@ -790,7 +658,7 @@ class TestVolumeList(TestVolume):
 
     columns = [
         'ID',
-        'Display Name',
+        'Name',
         'Status',
         'Size',
         'Attached to',
@@ -827,7 +695,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': False,
             'project_id': None,
             'user_id': None,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -870,7 +738,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': True,
             'project_id': self.project.id,
             'user_id': None,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -915,7 +783,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': True,
             'project_id': self.project.id,
             'user_id': None,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -958,7 +826,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': False,
             'project_id': None,
             'user_id': self.user.id,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -1002,7 +870,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': False,
             'project_id': None,
             'user_id': self.user.id,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -1045,7 +913,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': False,
             'project_id': None,
             'user_id': None,
-            'display_name': self.mock_volume.name,
+            'name': self.mock_volume.name,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -1088,7 +956,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': False,
             'project_id': None,
             'user_id': None,
-            'display_name': None,
+            'name': None,
             'status': self.mock_volume.status,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -1131,7 +999,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': True,
             'project_id': None,
             'user_id': None,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -1175,7 +1043,7 @@ class TestVolumeList(TestVolume):
             'all_tenants': False,
             'project_id': None,
             'user_id': None,
-            'display_name': None,
+            'name': None,
             'status': None,
         }
         self.volumes_mock.list.assert_called_once_with(
@@ -1186,7 +1054,7 @@ class TestVolumeList(TestVolume):
 
         collist = [
             'ID',
-            'Display Name',
+            'Name',
             'Status',
             'Size',
             'Type',
@@ -1248,7 +1116,7 @@ class TestVolumeList(TestVolume):
                 'status': None,
                 'project_id': None,
                 'user_id': None,
-                'display_name': None,
+                'name': None,
                 'all_tenants': False, }
         )
         self.assertEqual(datalist, tuple(data))
@@ -1263,6 +1131,42 @@ class TestVolumeList(TestVolume):
         self.assertRaises(argparse.ArgumentTypeError, self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_volume_list_backward_compatibility(self):
+        arglist = [
+            '-c', 'Display Name',
+        ]
+        verifylist = [
+            ('columns', ['Display Name']),
+            ('long', False),
+            ('all_projects', False),
+            ('name', None),
+            ('status', None),
+            ('marker', None),
+            ('limit', None),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        search_opts = {
+            'all_tenants': False,
+            'project_id': None,
+            'user_id': None,
+            'name': None,
+            'status': None,
+        }
+        self.volumes_mock.list.assert_called_once_with(
+            search_opts=search_opts,
+            marker=None,
+            limit=None,
+        )
+
+        self.assertIn('Display Name', columns)
+        self.assertNotIn('Name', columns)
+
+        for each_volume in data:
+            self.assertIn(self.mock_volume.name, each_volume)
+
 
 class TestVolumeMigrate(TestVolume):
 
@@ -1368,6 +1272,24 @@ class TestVolumeSet(TestVolume):
         # Get the command object to test
         self.cmd = volume.SetVolume(self.app, None)
 
+    def test_volume_set_property(self):
+        arglist = [
+            '--property', 'a=b',
+            '--property', 'c=d',
+            self.new_volume.id,
+        ]
+        verifylist = [
+            ('property', {'a': 'b', 'c': 'd'}),
+            ('volume', self.new_volume.id),
+            ('bootable', False),
+            ('non_bootable', False)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+        self.volumes_mock.set_metadata.assert_called_with(
+            self.new_volume.id, parsed_args.property)
+
     def test_volume_set_image_property(self):
         arglist = [
             '--image-property', 'Alpha=a',
