@@ -112,8 +112,8 @@ class CreateVolumeType(command.ShowOne):
         parser.add_argument(
             '--encryption-provider',
             metavar='<provider>',
-            help=_('Set the class that provides encryption support for '
-                   'this volume type (e.g "LuksEncryptor") (admin only) '
+            help=_('Set the encryption provider format for '
+                   'this volume type (e.g "luks" or "plain") (admin only) '
                    '(This option is required when setting encryption type '
                    'of a volume. Consider using other encryption options '
                    'such as: "--encryption-cipher", "--encryption-key-size" '
@@ -371,8 +371,8 @@ class SetVolumeType(command.Command):
         parser.add_argument(
             '--encryption-provider',
             metavar='<provider>',
-            help=_('Set the class that provides encryption support for '
-                   'this volume type (e.g "LuksEncryptor") (admin only) '
+            help=_('Set the encryption provider format for '
+                   'this volume type (e.g "luks" or "plain") (admin only) '
                    '(This option is required when setting encryption type '
                    'of a volume for the first time. Consider using other '
                    'encryption options such as: "--encryption-cipher", '
@@ -544,7 +544,7 @@ class UnsetVolumeType(command.Command):
             '--project',
             metavar='<project>',
             help=_('Removes volume type access to project (name or ID) '
-                   ' (admin only)'),
+                   '(admin only)'),
         )
         identity_common.add_project_domain_option_to_parser(parser)
         parser.add_argument(
